#!/bin/sh
# (c) Copyright 2001-2009. CodeWeavers, Inc.

vfolder_on=0
vfolder_user_menu=""
vfolder_user_desktop=""
vfolder_global_menu=""
vfolder_global_alt_menu=""
vfolder_global_desktop=""
vfolder_preferred_menu=""
vfolder_preferred_alt_menu=""
vfolder_preferred_desktop=""

kde_on=0
kde_version=""
kde_user_menu=""
kde_user_alt_menu=""
kde_user_desktop=""
kde_user_mime=""
kde_global_menu=""
kde_global_alt_menu=""
kde_global_desktop=""
kde_global_desktoplinks=""
kde_global_mime=""
kde_preferred_menu=""
kde_preferred_alt_menu=""
kde_preferred_desktop=""
kde_preferred_mime=""
kde_mime_path=""

gnome_on=0
gnome_version=""
gnome_user_menu=""
gnome_user_desktop=""
gnome_user_mime=""
gnome_user_application=""
gnome_global_menu=""
gnome_global_desktop=""
gnome_global_mime=""
gnome_global_application=""
gnome_preferred_menu=""
gnome_preferred_desktop=""
gnome_preferred_mime=""
gnome_preferred_application=""
gnome_mime_path=""

xdg_menu_on=0
xdg_user_menu=""
xdg_user_alt_menu=""
xdg_user_alt2_menu=""
xdg_global_menu=""
xdg_global_alt_menu=""
xdg_global_alt2_menu=""
xdg_preferred_menu=""
xdg_preferred_alt_menu=""
xdg_preferred_alt2_menu=""

xdg_mime_on=0
xdg_user_mime=""
xdg_global_mime=""
xdg_preferred_mime=""
xdg_data_dirs=""

dtop_on=0
dtop_user_desktop=""
dtop_user_alt_desktop=""
dtop_global_desktop=""
dtop_global_alt_desktop=""
dtop_preferred_desktop=""
dtop_preferred_alt_desktop=""

suse_on=0
suse_user_menu=""
suse_global_menu=""
suse_preferred_menu=""

debian_menu_on=0
debian_user_menu=""
debian_global_menu=""
debian_old_global_menu=""
debian_preferred_menu=""
debian_old_preferred_menu=""

debian_mime_on=0
debian_global_assoc=""
debian_preferred_assoc=""

cde_on=0
cde_user_dt=""
cde_user_wm=""
cde_global_dt=""
cde_global_wm=""
cde_preferred_dt=""
cde_preferred_wm=""

macosx_on=0

mailcap_on=0
mailcap_user_mailcap=""
mailcap_user_mime=""
mailcap_global_mailcap=""
mailcap_global_mime=""
mailcap_preferred_mailcap=""
mailcap_preferred_mime=""

desktop_assoc_ignore_list=""
desktop_menu_ignore_list=""

# Directories where to look for kde/gnome.
kde_global_dirs="/usr /opt/kde3 /opt/kde2 /opt/kde /usr/kde/3 /usr/kde"
kde_user_dirs="$HOME/.kde $HOME/.kde3 $HOME/.kde2"
kde_desktops="$HOME/KDesktop"
gnome_global_dirs="/usr/share /usr/share/gnome /opt/gnome2/share /opt/gnome/share /usr/X11R6/share/gnome /opt/gnome-1.4/share /usr/gnome/share"
gnome_user_dirs="$HOME/.gnome"


# ----- Options ------

error()
{
    name0=`basename "$0"`
    echo "$name0:error: " "$@" >&2
}

usage=""
quiet=0
scope="private"
menu=0
assoc=0

while [ $# -gt 0 ]
do
    case "$1" in
    --quiet)
        quiet=1
        shift
        ;;
    --scope)
        scope="$2"
        shift 2
        ;;
    --menu)
        menu=1
        shift
        ;;
    --assoc)
        assoc=1
        shift
        ;;
    --help|-h|-\?)
        usage=0
        shift
        ;;
    *)
        error "unknown option '$1'"
        usage=2
        shift
        ;;
    esac
done

if [ $menu -eq 0 -a $assoc -eq 0 ]
then
    menu=1
    assoc=1
fi

uid=`id | sed -e 's/^uid=\([0-9]*\)(.*/\1/'`
if [ "$scope" != "managed" -a "$scope" != "private" ]
then
    error "unknown scope '$scope'"
    usage=2
else
    if [ "$scope" = "managed" -a "$uid" != "0" ]
    then
        error "only root can use --scope managed"
        usage=2
    fi
fi

# Check that we can use $HOME and if not, unset it so buggy applications,
# like Ubuntu's kde-config, don't try to write to it
if [ -n "$HOME" ]
then
    if [ ! -d "$HOME" -o ! -w "$HOME" ]
    then
        unset HOME
    else
        owner=`ls -Llnd "$HOME" | sed -e 's/^[^ ]*  *[^ ]*  *\([0-9]*\)  *.*$/\1/'`
        if [ "$uid" != "$owner" -a "$owner" != "0" ]
        then
            # Probably an 'su' that did not update $HOME
            unset HOME
        fi
    fi
fi

if [ "$scope" != "managed" -a -z "$HOME" ]
then
    error "\$HOME must be set correctly and be writable for --scope private"
    usage=2
fi

if [ -n "$usage" ]
then
    name0=`basename "$0"`
    if [ $usage -ne 0 ]
    then
        error "try '$name0 --help' for more information"
        exit $usage
    fi
    cat <<EOF
Usage: $name0 [--quiet] [--scope SCOPE] [--menu|--assoc]

Scans for menu and association systems and returns their locations.

Options:
  --quiet       Don't print the shell commands that set all the variables
  --scope SCOPE If set to 'managed' return the system wide locations as the
                preferred location, otherwise return the per-user location
  --menu        Only scan for menuing systems
  --assoc       Only scan for association systems
EOF
    exit 0
fi


# ----- VFolder ------

if [ $menu -ne 0 ]
then
    for file in /etc/X11/desktop-menus/applications.menu \
                /etc/opt/gnome/gnome-vfs-2.0/vfolders/applications.vfolder-info \
                /etc/opt/gnome/gnome-vfs-2.0/vfolders/applications-all-users.vfolder-info \
                /etc/opt/gnome2/gnome-vfs-2.0/vfolders/applications.vfolder-info \
                /etc/opt/gnome2/gnome-vfs-2.0/vfolders/applications-all-users.vfolder-info \
                /etc/gnome-vfs-2.0/vfolders/applications.vfolder-info \
                /etc/gnome-vfs-2.0/vfolders/applications-all-users.vfolder-info \
                /etc/gnome/gnome-vfs-2.0/vfolders/applications.vfolder-info \
                /etc/gnome/gnome-vfs-2.0/vfolders/applications-all-users.vfolder-info
    do
        if [ -f "$file" ]
        then
            vfolder_global_menu="$file"
            break
        fi
    done

    vfolder_global_alt_menu="/etc/opt/gnome/gnome-vfs-2.0/vfolders/suse-menu.vfolder-info"
    if [ ! -f "$vfolder_global_alt_menu" ]
    then
        vfolder_global_alt_menu=""
    fi

    if [ -n "$vfolder_global_menu" ]
    then
        vfolder_on=1
        vfolder_global_desktop="/usr/share/gnome/desktop"
        vfolder_user_menu="$HOME/.gnome2/vfolders/applications.vfolder-info"
        vfolder_user_desktop="$HOME/.gnome-desktop"
    fi
fi

# ----- KDE ------

safe_kde_config()
{
    decoy=""
    if [ -z "$HOME" ]
    then
        decoy="/tmp/decoy.$$"
        HOME="$decoy"
        touch "$decoy"
    fi
    kde-config "$@"
    rc=$?
    [ -z "$decoy" ] || rm "$decoy"
    return $rc
}

safe_kde_config()
{
    decoy=""
    if [ -z "$HOME" ]
    then
        decoy="/tmp/decoy.$$"
        HOME="$decoy"
        touch "$decoy"
    fi
    kde-config "$@"
    rc=$?
    [ -z "$decoy" ] || rm "$decoy"
    return $rc
}

# kde-config is not in $PATH when installing an RPM package on SuSE
# And it is not in root's $PATH on Solaris
PATH="$PATH:/opt/kde3/bin:/opt/sfw/kde/bin"

# First try to use kde-config.
type kde-config >/dev/null 2>&1
if [ $? = 0 ]
then
    kde_apps=`safe_kde_config --path apps 2>/dev/null`
    rc=$?
    kde_user_prefix=`safe_kde_config --localprefix 2>/dev/null`
    rc=`expr $rc + $?`
    kde_global_prefix=`safe_kde_config --prefix 2>/dev/null`
    rc=`expr $rc + $?`
    kde_user_config=`safe_kde_config --path config 2>/dev/null`
    rc=`expr $rc + $?`
    if [ $assoc -ne 0 ]
    then
        kde_mime_path=`safe_kde_config --path mime 2>/dev/null`
        rc=`expr $rc + $?`
        kde_mime_path=`echo "$kde_mime_path" | sed -e 's%/:%:%g' -e 's%/$%%'`
    fi

    if [ $rc -eq 0 -a -n "$kde_user_prefix" -a -n "$kde_global_prefix" ]
    then
        kde_on=1

        # Many kde-config commands return multiple paths so we have to pick
        # the right one (user/global).
        kde_user_config=`echo "$kde_user_config" | tr ':' '\n' | egrep "^$kde_user_prefix" | head -n 1`
        if [ -z "$kde_user_config" ]
        then
            # kde-config --localprefix is broken on Gentoo so that we don't
            # find any match. Use $HOME instead.
            kde_user_prefix="$HOME/.kde"
            kde_user_config=`echo "$kde_user_config" | tr ':' '\n' | egrep "^$kde_user_prefix" | head -n 1`
        fi

        # We need these for the KDE MIME types too
        kde_user_menu=`echo "$kde_apps" | tr ':' '\n' | egrep "^$kde_user_prefix" | sed -e '2,$ d' -e 's%/$%%'`
        # Try to extract desktop location from share/config/kdeglobals.
        kde_user_desktop=`sed -e 's/^Desktop=//' -e 't strip' -e 'd' -e ': strip' -e 's%/$%%' "$kde_user_config/kdeglobals" 2>/dev/null`
        kde_user_desktop=`eval "echo $kde_user_desktop"`
        if [ -z "$kde_user_desktop" ]
        then
            # Try to find the desktop dir.
            for kde_desktop in $kde_desktops
            do
                if [ -d "$kde_desktop" ]
                then
                    kde_user_desktop="$kde_desktop"
                    break
                fi
            done
        fi
        kde_global_menu=`echo "$kde_apps" | tr ':' '\n' | egrep "^$kde_global_prefix" | sed -e '2,$ d' -e 's%/$%%'`
        kde_global_desktop="$kde_global_prefix/share/apps/kdesktop/Desktop"

        if [ $assoc -ne 0 ]
        then
            kde_user_mime=""
            kde_local_mime=""
            kde_global_mime=""
            original_ifs="$IFS"
            IFS=":"
            for dir in $kde_mime_path
            do
                IFS="$original_ifs"
                if [ -z "$kde_user_mime" ] && echo "$dir" | egrep "^$kde_user_prefix" >/dev/null
                then
                    kde_user_mime="$dir"
                elif [ -z "$kde_local_mime" ] && echo "$dir" | egrep "^/usr/local/" >/dev/null
                then
                    kde_local_mime="$dir"
                elif [ -z "$kde_global_mime" ] && echo "$dir" | egrep "^$kde_global_prefix" >/dev/null
                then
                    kde_global_mime="$dir"
                fi
            done
            if [ -z "$kde_global_mime" ]
            then
                kde_global_mime="$kde_local_mime"
            fi
        fi
    fi
fi

# kde-config isn't installed.
if [ $kde_on = 0 ]
then
    # So, try to find the global KDE directory.
    for dir in $kde_global_dirs
    do
        if [ -d "$dir/share/mimelnk" ]
        then
            kde_global_prefix="$dir"
            kde_on=1
            break
        fi
    done

    # We found KDE global directory.
    if [ $kde_on = 1 ]
    then
        # Try to find the user KDE dir.
        for dir in $kde_user_dirs
        do
            if [ -d "$dir/share/mimelnk" ]
            then
                kde_user_prefix="$dir"
                break
            fi
        done
        # If no KDE user dir found, use $HOME/.kde as default.
        if [ -z "$kde_user_prefix" ]
        then
            kde_user_prefix="$HOME/.kde"
        fi
                
        # We need these for the KDE MIME types too
        kde_user_menu="$kde_user_prefix/share/applnk"
        kde_user_desktop=`sed -e 's/^Desktop=//' -e 't strip' -e 'd' -e ': strip' -e 's%/$%%' "$kde_user_prefix/share/config/kdeglobals" 2>/dev/null`
        kde_user_desktop=`eval "echo $kde_user_desktop"`
        if [ -z "$kde_user_desktop" ]
        then
            for kde_desktop in $kde_desktops
            do
                if [ -d "$kde_desktop" ]
                then
                    kde_user_desktop="$kde_desktop"
                    break
                fi
            done
        fi
        kde_global_menu="$kde_global_prefix/share/applnk"
        kde_global_desktop="$kde_global_prefix/share/apps/kdesktop/Desktop"

        if [ $assoc -ne 0 ]
        then
            kde_user_mime="$kde_user_prefix/share/mimelnk"
            kde_global_mime="$kde_global_prefix/share/mimelnk"
            kde_mime_path=""
            for dir in $kde_global_dirs $kde_user_dirs
            do
                if [ -d "$dir/share/mimelnk" ]
                then
                    if [ -n "$kde_mime_path" ]
                    then
                        kde_mime_path="$kde_mime_path:"
                    fi
                    kde_mime_path="$kde_mime_path$dir/share/mimelnk"
                fi
            done
        fi
    fi
fi
if [ $kde_on -eq 1 ]
then
    type kde-config >/dev/null 2>&1
    if [ $? = 0 ]
    then
        kvertool="safe_kde_config"
    else
        type konqueror >/dev/null 2>&1
        if [ $? = 0 ]
        then
            kvertool="konqueror"
        fi
    fi
    if [ -n "$kvertool" ]
    then
        kde_version=`$kvertool --version | sed -e 's/^KDE: \([0-9.]*\).*$/\1/' -e 't' -e 'd'`
    fi

    if [ -d "${kde_global_desktop}Links" ]
    then
        kde_global_desktoplinks="${kde_global_desktop}Links"
    fi
fi


# ------ Debian menus ------

# We need the tweaked KDE menus for the KDE MIME types too
for dir in "/usr/share/menu" "/usr/lib/menu"
do
    if [ -d "$dir" ]
    then
        type update-menus >/dev/null 2>&1
        if [ $? = 0 ]
        then
            debian_menu_on=1
            debian_user_menu="$HOME/.menu"
            debian_global_menu="$dir"
            if [ "$dir" != "/usr/lib/menu" -a -d "/usr/lib/menu" ]
            then
                debian_old_global_menu="/usr/lib/menu"
            fi

            # If Debian menus are in use, kde-config --path apps returns paths
            # used by the Debian menu system. We have to guess where the
            # "original" menu trees are.
            if [ -n "$kde_apps" ]
            then
                # Use the prefixes returned by kde-config.
                kde_user_menu="$kde_user_prefix/share/applnk"
                kde_global_menu="$kde_global_prefix/share/applnk"
            fi
        fi
        break
    fi
done


# ------ Debian mime settings -----

if [ $assoc -ne 0 -a -d "/usr/lib/mime/packages" ]
then
    type update-mime >/dev/null 2>&1
    if [ $? = 0 ]
    then
        debian_mime_on=1
        debian_global_assoc=/usr/lib/mime/packages
    fi
fi


# ----- Gnome ------

# gnome-config / gnome-panel are not in $PATH when installing
# an RPM package on SuSE
PATH="$PATH:/opt/gnome/bin:/opt/gnome2/bin"

# Try to use gnome-config to find the global gnome dir.
type gnome-config >/dev/null 2>&1
if [ $? = 0 ]
then
    gnome_on=1
    gnome_datadir=`gnome-config --datadir`
fi

# If gnome-config isn't installed try some common locations.
if [ $gnome_on = 0 ]
then
    for gnome_datadir in $gnome_global_dirs
    do
        if [ -d "$gnome_datadir/mime-info" ]
        then
            gnome_on=1
            break
        fi
    done
fi

# Gnome seems to be installed so set the paths.
if [ $gnome_on = 1 ]
then
    if [ $menu -ne 0 ]
    then
        # On SuSE 8.1 the legacy menus are not in gnome_datadir
        for dir in $gnome_datadir $gnome_global_dirs
        do
            if [ -d "$dir/gnome/apps" ]
            then
                gnome_global_menu="$dir/gnome/apps"
                break
            fi
        done
        gnome_user_menu="$HOME/.gnome/apps"
        gnome_global_desktop="$gnome_datadir/gnome/desktop"
        gnome_user_desktop="$HOME/.gnome-desktop"
    fi
    if [ $assoc -ne 0 ]
    then
        gnome_global_mime="$gnome_datadir/mime-info"
        gnome_global_application="$gnome_datadir/application-registry"
        gnome_user_mime="$HOME/.gnome/mime-info"
        gnome_user_application="$HOME/.gnome/application-info"
        gnome_mime_path=""
        for dir in $gnome_global_dirs $gnome_user_dirs
        do
            if [ -d "$dir/mime-info" ]
            then
                if [ -n "$gnome_mime_path" ]
                then
                    gnome_mime_path="$gnome_mime_path:"
                fi
                gnome_mime_path="$gnome_mime_path$dir/mime-info"
            fi
        done
    fi
fi

# We really need a default value for gnome_shversion
gnome_shversion="204"
if [ $gnome_on -eq 1 -o $vfolder_on -eq 1 ]
then
    # Notes:
    # - Avoid gnome-session because it puts up an interactive dialog if it
    #   cannot resolve the hostname. gnome-panel's version is more relevant
    #   and should be close anyway."$ENV{PATH}:/opt/gnome2/bin"
    type gnome-panel >/dev/null 2>&1
    if [ $? = 0 ]
    then
        gnome_version=`gnome-panel --version | sed -e 's/^.*gnome-panel *//'`
        gnome_shversion=`echo $gnome_version | sed -e 's/^\([0-9]*\.[0-9]*\).*$/\1/' -e 's/\.\([0-9]\)$/0\1/' -e 's/\.//'`
    else
        # This is a pre-2.0 Gnome version
        gnome_version="1.x"
        gnome_shversion="109"
    fi
fi


# ----- XDG ------
if [ -z "$XDG_DATA_DIRS" ]
then
    xdg_data_dirs="/usr/local/share:/usr/share"
else
    xdg_data_dirs="$XDG_DATA_DIRS"
fi

# See if we can find XDG mime info
original_ifs="$IFS"
IFS=":"
for dir in $xdg_data_dirs
do
    IFS="$original_ifs"
    if [ -f "$dir/mime/globs" -a -d "$dir/mime/packages" -a \
         -d "$dir/applications" ]
    then
        xdg_mime_on=1
        xdg_global_mime="$dir"
        if [ -z "$XDG_DATA_HOME" ]
        then
            xdg_user_mime="$HOME/.local/share"
        else
            xdg_user_mime="$XDG_DATA_HOME"
        fi
        break
    fi
done

# See if we can find XDG menu info
xdg_menus=":"
xdg_add_menu()
{
    global_menu="$1"

    # Check if this is a duplicate
    if echo "$xdg_menus" | egrep ":$global_menu:" >/dev/null
    then
        return
    fi
    xdg_menus="$xdg_menus$global_menu:"

    # Compute user_menu
    userdir="$XDG_CONFIG_HOME"
    [ -n "$userdir" ] || userdir="$HOME/.config"
    userdir="$userdir/menus"

    # Add the menu
    xdg_menu_on=`expr $xdg_menu_on + 1`
    eval "xdg_global_menu$xdg_menu_on=\"$global_menu\""
    eval "xdg_user_menu$xdg_menu_on=\"$userdir\"/`basename \"$global_menu\"`"
    if [ "$scope" = "managed" ]
    then
        eval "xdg_preferred_menu$xdg_menu_on=\"$global_menu\""
    else
        eval "xdg_preferred_menu$xdg_menu_on=\"\$xdg_user_menu$xdg_menu_on\""
    fi
}

xdg_scan_menu_dir()
{
    menudir="$1"

    found_match=0
    for match in "$menudir"/*-applications.menu "$menudir"/*-applications-merged
    do
        if [ -f "$match" -o -d "$match" ]
        then
            xdg_menu=`echo "$match" | sed -e 's/-merged$/.menu/'`
            xdg_add_menu "$xdg_menu"
            found_match=1
        fi
    done
    return $found_match
}

original_ifs="$IFS"
IFS=":"
for dir in $XDG_CONFIG_DIRS /usr/local/etc/xdg /etc/xdg
do
    IFS="$original_ifs"
    if [ -n "$dir" -a -d "$dir/menus" ]
    then
        xdg_scan_menu_dir "$dir/menus"

        # If it looks like an XDG menu directory, then always add
        # applications.menu, even if it does not exist, because it may be
        # needed if the user installed a custom KDE version somewhere
        if [ $? -eq 1 -o -f "$dir/menus/applications.menu" -o -d "$dir/menus/applications-merged" ]
        then
            xdg_add_menu "$dir/menus/applications.menu"
        fi
    fi
done

# On Ubuntu 8.04, KDE 4 puts its XDG menus in a strange place
xdg_scan_menu_dir "/usr/lib/kde4/etc/xdg/menus"


# ----- XDG / KDE / Gnome desktop ------

# If ~/Desktop does not exist, then assume this is because the user
# wants it so and don't clutter his $HOME directory
if [ -d "$HOME/Desktop" -a \( $xdg_menu_on != 0 -o $kde_on = 1 -o $gnome_on = 1 \) ]
then
    dtop_on=1
    dtop_user_desktop="$HOME/Desktop"
fi


# ----- XDG User directories desktop ------

if [ -f "$HOME/.config/user-dirs.dirs" ]
then
    . "$HOME/.config/user-dirs.dirs"
    if [ -n "$XDG_DESKTOP_DIR" -a "$XDG_DESKTOP_DIR" != "$dtop_user_desktop" -a -d "$XDG_DESKTOP_DIR" ]
    then
        dtop_on=1
        dtop_user_alt_desktop="$XDG_DESKTOP_DIR"
    fi
fi


# ----- SuSE ------

# SuSE (8.0, 8.1 and 8.2) - KDE
# We might need the special SuSE menus for the KDE MIME types too
if [ -d "/etc/X11/susewm/AddEntrys/SuSE" -a -f "/sbin/conf.d/SuSEconfig.susewm" ]
then
    suse_on=1
    suse_global_menu="/etc/X11/susewm/AddEntrys/SuSE"
    suse_user_menu="$HOME/.kde/share/applnk/SuSE"
fi


# ----- CDE ------

for dir in /etc/dt /usr/dt
do
    if [ -d "$dir/appconfig/types/C" ]
    then
        cde_global_dt="$dir/appconfig/types/C"
        cde_global_wm="$dir/config/C/wsmenu"
        cde_on=1
        break
    fi
done

if [ $cde_on ]
then
    cde_user_dt="$HOME/.dt/types"
    cde_user_wm="$HOME/.dt/wsmenu"
fi

# ----- Mailcap -----

if [ $assoc -ne 0 ]
then
    mailcap_on=1

    mailcap_user_mailcap="$HOME/.mailcap"
    mailcap_user_mime="$HOME/.mime.types"

    # Note: RFC 1524 mentions the MAILCAPS environment variable
    # but it is unclear that we need it right now
    if [ -f "/usr/dt/appconfig/netscape/etc/mailcap" ]
    then
        mailcap_global_mailcap="/usr/dt/appconfig/netscape/etc/mailcap"
        mailcap_global_mime="/usr/dt/appconfig/netscape/etc/mime.types"
    else
        mailcap_global_mailcap="/etc/mailcap"
        mailcap_global_mime="/etc/mime.types"
    fi
fi


# ----- Mac OS X ------

if [ -f "/System/Library/DTDs/PropertyList.dtd" ]
then
    macosx_on=1
fi


# ----- Now we do distribution specific kludges^H^H^H^H^H^H^Hproprietary extensions


# RedHat 8.0 - KDE mods
#   RedHat has hacked KDE to that it doesn't respond to the 'normal' applnk
#   directories any more; it responds to ones named 'applnk-redhat'.
# We need the tweaked KDE menus for the KDE MIME types too
if [ $kde_on = 1 -a -d "/usr/share/applnk-redhat" ]
then
    if [ "$kde_global_menu" != "/usr/share/applnk-redhat" ]
    then
        kde_global_alt_menu="/usr/share/applnk-redhat"
    fi
    if [ "$kde_user_menu" != "$HOME/.kde/share/applnk-redhat" ]
    then
        kde_user_alt_menu="$HOME/.kde/share/applnk-redhat"
    fi
fi

# Set the preferred paths depending on who ran the script (root/regular user).
if [ "$scope" = "managed" ]
then
    if [ $menu -ne 0 ]
    then
        vfolder_preferred_menu="$vfolder_global_menu"
        vfolder_preferred_alt_menu="$vfolder_global_alt_menu"
        vfolder_preferred_desktop="$vfolder_global_desktop"
        gnome_preferred_menu="$gnome_global_menu"
        gnome_preferred_desktop="$gnome_global_desktop"
        suse_preferred_menu="$suse_global_menu"
        #xdg_preferred_menu*
        #dtop_preferred_desktop
        #dtop_preferred_alt_desktop
    fi

    # We need these for the KDE MIME types on Mandrake too
    debian_preferred_menu="$debian_global_menu"
    debian_old_preferred_menu="$debian_old_global_menu"

    # We need these for the KDE MIME types too
    kde_preferred_menu="$kde_global_menu"
    kde_preferred_alt_menu="$kde_global_alt_menu"
    kde_preferred_desktop="$kde_global_desktop"

    if [ $assoc -ne 0 ]
    then
        kde_preferred_mime="$kde_global_mime"
        gnome_preferred_mime="$gnome_global_mime"
        gnome_preferred_application="$gnome_global_application"
        xdg_preferred_mime="$xdg_global_mime"
        debian_preferred_assoc="$debian_global_assoc"
        mailcap_preferred_mailcap="$mailcap_global_mailcap"
        mailcap_preferred_mime="$mailcap_global_mime"
    fi
    cde_preferred_dt="$cde_global_dt"
    cde_preferred_wm="$cde_global_wm"
else
    if [ $menu -ne 0 ]
    then
        vfolder_preferred_menu="$vfolder_user_menu"
        #vfolder_preferred_alt_menu
        vfolder_preferred_desktop="$vfolder_user_desktop"
        gnome_preferred_menu="$gnome_user_menu"
        gnome_preferred_desktop="$gnome_user_desktop"
        suse_preferred_menu="$suse_user_menu"
        #xdg_preferred_menu*
        dtop_preferred_desktop="$dtop_user_desktop"
        dtop_preferred_alt_desktop="$dtop_user_alt_desktop"
    fi

    # We need these for the KDE MIME types on Mandrake too
    debian_preferred_menu="$debian_user_menu"

    # We need these for the KDE MIME types too
    kde_preferred_menu="$kde_user_menu"
    kde_preferred_alt_menu="$kde_user_alt_menu"
    kde_preferred_desktop="$kde_user_desktop"

    if [ $assoc -ne 0 ]
    then
        kde_preferred_mime="$kde_user_mime"
        gnome_preferred_mime="$gnome_user_mime"
        gnome_preferred_application="$gnome_user_application"
        xdg_preferred_mime="$xdg_user_mime"
        #debian_preferred_assoc
        mailcap_preferred_mailcap="$mailcap_user_mailcap"
        mailcap_preferred_mime="$mailcap_user_mime"
    fi
    cde_preferred_dt="$cde_user_dt"
    cde_preferred_wm="$cde_user_wm"
fi


# Blacklist the redundant / mutually incompatible menuing systems

if [ $xdg_menu_on != 0 ]
then
    if [ -n "$kde_version" ]
    then
        kde_shversion=`echo $kde_version | sed -e 's/^\([0-9]*\.[0-9]*\).*$/\1/' -e 's/\.\([0-9]\)$/0\1/' -e 's/\.//'`
    else
        kde_shversion="302"
    fi
    if [ -n "$kde_preferred_menu" -a $kde_shversion -gt 301 ]
    then
        # If we have XDG menus, then KDE menus are redundant (and may create
        # duplicate menu entries if enabled), except on SuSE 9.0 which ships
        # with KDE 3.1
        desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuKDE$kde_preferred_menu"
    fi
    if [ -n "$gnome_preferred_menu" ]
    then
        # If we have XDG menus, then Gnome menus are redundant
        desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuGnome$gnome_preferred_menu"
    fi
fi

if [ $debian_menu_on -eq 1 ]
then
    if [ -f "/etc/xandros-desktop-version" ]
    then
        # Don't create Debian menus on Xandros
        desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuDebian$debian_preferred_menu"
    elif [ -f "/etc/debian_version" -a "$gnome_preferred_menu" = "$HOME/.gnome/apps" ]
    then
        # On Debian 3.0 we should create the Gnome menus ourselves
        # but on Debian 3.1 the Debian menuing system will do it for us
        grep 3.0 /etc/debian_version >/dev/null
        if [ $? -ne 0 ]
        then
            desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuGnome$gnome_preferred_menu"
        fi
    elif [ "$scope" = "managed" -a -f "/etc/mandriva-release" ] && \
        egrep "release  *2006\\.0[^0-9]" "/etc/mandriva-release" >/dev/null 2>&1
    then
        # Don't create global Debian menus on Mandriva 2006.0
        # as this causes duplicate menu entries
        desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuDebian$debian_preferred_menu"

    fi
fi

# Blacklist the redundant / mutually incompatible desktop icon systems

if [ "$scope" = "private" ]
then
    if [ -n "$kde_preferred_desktop" -a "$kde_preferred_desktop" = "$dtop_preferred_desktop" ]
    then
        desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuKDE$kde_preferred_desktop"
    fi

    if [ $gnome_shversion -ge 0204 ]
    then
        if [ -n "$gnome_preferred_desktop" -a ! -d "$gnome_preferred_desktop" ]
        then
            # Starting with version 2.4, Gnome does not use the old desktop
            # location anymore
            desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuGnome$gnome_preferred_desktop"
        fi
        if [ -n "$vfolder_preferred_desktop" -a ! -d "$vfolder_preferred_desktop" ]
        then
            # And starting with version 2.4, Gnome's VFolder implementation
            # does not use the old desktop location either
            desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuVFDesktop$vfolder_preferred_desktop"
        fi
    fi
fi

if [ -n "$vfolder_preferred_desktop" -a "$vfolder_preferred_desktop" = "$gnome_preferred_desktop" ]
then
    # If the Gnome desktop (legacy-mixed encoding) matches the VFolder 
    # desktop (UTF-8), then give the priority to VFolder
    desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuGnome$gnome_preferred_desktop"
fi

desktop_menu_ignore_list=`echo "$desktop_menu_ignore_list" | sed -e 's%//*%/%g'`

if [ $cde_on -eq 1 -a -x "/usr/dt/lib/dtobsolete" ]
then
    # On Solaris >= 10u5 invoking CDE pops up a deprecation warning
    desktop_menu_ignore_list="$desktop_menu_ignore_list:CXMenuCDE$cde_preferred_wm"
fi


# Blacklist the redundant / mutually incompatible association systems

if [ $debian_menu_on -eq 1 -a ! -d "/usr/share/applnk-mdk" ]
then
    # Creating Mandrake associations on other systems is redundant and
    # usually creates an ugly '.hidden' folder.
    desktop_assoc_ignore_list="$desktop_assoc_ignore_list:CXAssocMandrake$debian_preferred_menu"
fi

if [ $xdg_mime_on -eq 1 -a -n "$kde_preferred_mime" ]
then
    # Systems that support XDG associations don't need the KDE ones...
    # except on Fedora Core!
    if [ ! -f /etc/redhat-release ]
    then
        ignore=1
    elif egrep "^Fedora Core " /etc/redhat-release >/dev/null
    then
        ignore=0
    else
        ignore=1
    fi
    if [ $ignore -eq 1 ]
    then
        desktop_assoc_ignore_list="$desktop_assoc_ignore_list:CXAssocKDE$kde_preferred_menu"
    fi
fi

if [ -n "$debian_preferred_assoc" -a $mailcap_on -eq 1 ]
then
    # Debian associations replace the Mailcap ones
    desktop_assoc_ignore_list="$desktop_assoc_ignore_list:CXAssocMailcap$mailcap_preferred_mailcap"
fi

desktop_assoc_ignore_list=`echo "$desktop_assoc_ignore_list" | sed -e 's%//*%/%g'`


# Print the paths.
if [ $quiet -eq 0 ]
then
    echo preferred_scope=$scope

    echo vfolder_on=$vfolder_on
    echo kde_on=$kde_on
    echo kde_version=$kde_version
    echo gnome_on=$gnome_on
    echo gnome_version=$gnome_version
    echo xdg_menu_on=$xdg_menu_on
    echo xdg_mime_on=$xdg_mime_on
    echo suse_on=$suse_on
    echo dtop_on=$dtop_on
    echo cde_on=$cde_on
    echo macosx_on=$macosx_on

    if [ "$scope" != "managed" ]
    then
        echo cde_user_dt=$cde_user_dt
        echo cde_user_wm=$cde_user_wm
    fi
    echo cde_global_dt=$cde_global_dt
    echo cde_global_wm=$cde_global_wm
    echo cde_preferred_dt=$cde_preferred_dt
    echo cde_preferred_wm=$cde_preferred_wm

    if [ $menu -ne 0 ]
    then
        if [ "$scope" != "managed" ]
        then
            echo vfolder_user_menu=$vfolder_user_menu
            echo vfolder_user_desktop=$vfolder_user_desktop
        fi
        echo vfolder_global_menu=$vfolder_global_menu
        echo vfolder_global_alt_menu=$vfolder_global_alt_menu
        echo vfolder_global_desktop=$vfolder_global_desktop
        echo vfolder_preferred_menu=$vfolder_preferred_menu
        echo vfolder_preferred_alt_menu=$vfolder_preferred_alt_menu
        echo vfolder_preferred_desktop=$vfolder_preferred_desktop
    fi

    # We need these for the KDE MIME types too
    if [ "$scope" != "managed" ]
    then
        echo kde_user_menu=$kde_user_menu
        echo kde_user_alt_menu=$kde_user_alt_menu
        echo kde_user_desktop=$kde_user_desktop
    fi
    echo kde_global_menu=$kde_global_menu
    echo kde_global_alt_menu=$kde_global_alt_menu
    echo kde_global_desktop=$kde_global_desktop
    echo kde_global_desktoplinks=$kde_global_desktoplinks
    echo kde_preferred_menu=$kde_preferred_menu
    echo kde_preferred_alt_menu=$kde_preferred_alt_menu
    echo kde_preferred_desktop=$kde_preferred_desktop

    if [ $menu -ne 0 ]
    then
        if [ "$scope" != "managed" ]
        then
            echo gnome_user_menu=$gnome_user_menu
            echo gnome_user_desktop=$gnome_user_desktop
            echo suse_user_menu=$suse_user_menu
        fi
        echo gnome_global_menu=$gnome_global_menu
        echo gnome_global_desktop=$gnome_global_desktop
        echo gnome_preferred_menu=$gnome_preferred_menu
        echo gnome_preferred_desktop=$gnome_preferred_desktop
        echo suse_global_menu=$suse_global_menu
        echo suse_preferred_menu=$suse_preferred_menu
    fi

    if [ $menu -ne 0 ]
    then
        i=1
        while [ $i -le $xdg_menu_on ]
        do
            if [ "$scope" != "managed" ]
            then
                eval "echo xdg_user_menu$i=\$xdg_user_menu$i"
            fi
            eval "echo xdg_global_menu$i=\$xdg_global_menu$i"
            eval "echo xdg_preferred_menu$i=\$xdg_preferred_menu$i"
            i=`expr $i + 1`
        done
        if [ "$scope" != "managed" ]
        then
            echo dtop_user_desktop=$dtop_user_desktop
            echo dtop_user_alt_desktop=$dtop_user_alt_desktop
        fi
        echo dtop_preferred_desktop=$dtop_preferred_desktop
        echo dtop_preferred_alt_desktop=$dtop_preferred_alt_desktop
    fi

    # We need these for the KDE MIME types on Mandrake too
    echo debian_menu_on=$debian_menu_on
    if [ "$scope" != "managed" ]
    then
         echo debian_user_menu=$debian_user_menu
    fi
    echo debian_global_menu=$debian_global_menu
    echo debian_old_global_menu=$debian_old_global_menu
    echo debian_preferred_menu=$debian_preferred_menu
    echo debian_old_preferred_menu=$debian_old_preferred_menu

    if [ $assoc -ne 0 ]
    then
        echo kde_global_mime=$kde_global_mime
        echo kde_preferred_mime=$kde_preferred_mime
        echo kde_mime_path=$kde_mime_path

        echo gnome_global_mime=$gnome_global_mime
        echo gnome_global_application=$gnome_global_application
        echo gnome_preferred_mime=$gnome_preferred_mime
        echo gnome_preferred_application=$gnome_preferred_application
        echo gnome_mime_path=$gnome_mime_path

        echo xdg_global_mime=$xdg_global_mime
        echo xdg_preferred_mime=$xdg_preferred_mime
        echo xdg_data_dirs=$xdg_data_dirs

        echo debian_mime_on=$debian_mime_on
        echo debian_global_assoc=$debian_global_assoc
        echo debian_preferred_assoc=$debian_preferred_assoc

        echo mailcap_on=$mailcap_on
        echo mailcap_global_mailcap=$mailcap_global_mailcap
        echo mailcap_global_mime=$mailcap_global_mime
        echo mailcap_preferred_mailcap=$mailcap_preferred_mailcap
        echo mailcap_preferred_mime=$mailcap_preferred_mime
        if [ "$scope" != "managed" ]
        then
            echo kde_user_mime=$kde_user_mime

            echo gnome_user_mime=$gnome_user_mime
            echo gnome_user_application=$gnome_user_application

            echo xdg_user_mime=$xdg_user_mime

            echo mailcap_user_mailcap=$mailcap_user_mailcap
            echo mailcap_user_mime=$mailcap_user_mime
        fi
    fi

    echo desktop_assoc_ignore_list=$desktop_assoc_ignore_list
    echo desktop_menu_ignore_list=$desktop_menu_ignore_list
fi
